﻿using System;
using MySqlConnector;
using Stimulsoft.Base;
using Stimulsoft.Base.Localization;
using Stimulsoft.Report.Dictionary;
using Stimulsoft.Report.Dictionary.Design;

namespace Reports.Stimulsoft_Data_Adapter
{
	[System.ComponentModel.TypeConverter(typeof(StiSqlDatabaseConverter))]
	public class MySqlConnectorDatabase : StiSqlDatabase
	{
		#region Properties
		public override string ServiceName => StiLocalization.Get("Database", "DatabaseMySQL");

		protected override string DataAdapterType => "Reports.Stimulsoft_Data_Adapter.MySqlConnectorAdapterService";
		#endregion

		#region Methods
		/// <summary>
		/// Returns new data connector for this database.
		/// </summary>
		/// <returns>Created connector.</returns>
		public override StiDataConnector CreateConnector(string connectionString = null)
		{
			return StiMySqlConnector.Get(this.ConnectionString);
		}

		public override void RegData(StiDictionary dictionary, bool loadData)
		{
			var adapter = GetDataAdapter();
			if (adapter == null)
				throw new Exception($"Database {GetType()} not found");

			adapter.CreateConnectionInDataStore(dictionary, this);
		}

		protected override Type GetDataAdapterType()
		{
			return typeof(MySqlConnectorAdapterService);
		}

		public override StiSqlSource CreateDataSource(string nameInSource, string name)
		{
			return new MySqlConnectorSource(nameInSource, name);
		}

		#endregion

		/// <summary>
		/// Creates a new object of the type MySqlConnectorDatabase.
		/// </summary>
		public MySqlConnectorDatabase()
			: this(string.Empty, string.Empty)
		{
		}

		/// <summary>
		/// Creates a new object of the type MySqlConnectorDatabase.
		/// </summary>
		public MySqlConnectorDatabase(string name, string connectionString)
			: base(name, connectionString)
		{
		}

		/// <summary>
		/// Creates a new object of the type MySqlConnectorDatabase.
		/// </summary>
		public MySqlConnectorDatabase(string name, string alias, string connectionString)
			: base(name, alias, connectionString)
		{
		}

		/// <summary>
		/// Creates a new object of the type MySqlConnectorDatabase.
		/// </summary>
		public MySqlConnectorDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword)
			: base(name, alias, connectionString, promptUserNameAndpassword)
		{
		}

		/// <summary>
		/// Creates a new object of the type MySqlConnectorDatabase.
		/// </summary>
		public MySqlConnectorDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword, string key)
			: base(name, alias, connectionString, promptUserNameAndpassword, key)
		{
		}
	}
}
