﻿using System;
using MySqlConnector;
using Stimulsoft.Base;
using Stimulsoft.Base.Localization;
using Stimulsoft.Report;
using Stimulsoft.Report.Dictionary;
using Stimulsoft.Report.Dictionary.Design;

namespace Reports.Stimulsoft_Data_Adapter
{
	[System.ComponentModel.TypeConverter(typeof(StiSqlSourceConverter))]
	public class MySqlConnectorSource : StiSqlSource
	{
		#region Methods
		protected override Type ConvertDbTypeToTypeInternal(int sqlType)
		{
			MySqlDbType dbType = (MySqlDbType)sqlType;
			switch (dbType)
			{
				case MySqlDbType.Int64: //BigInt
				case MySqlDbType.Int32: //Int
				case MySqlDbType.Int16: //SmallInt
				case MySqlDbType.Byte: //TinyInt
					return typeof(Int64);

				case MySqlDbType.Decimal:
					return typeof(decimal);

				case MySqlDbType.Double:
				case MySqlDbType.Float:
					return typeof(double);

				case MySqlDbType.Date:
				case MySqlDbType.DateTime:
					//case MySqlType.Newdate:
					return typeof(DateTime);

				default:
					return typeof(string);
			}
		}

		public override Type GetParameterTypesEnum()
		{
			return typeof(MySqlDbType);
		}

		public override StiDataParameter AddParameter()
		{
			var parameter = new StiDataParameter(
				StiLocalization.Get("PropertyMain", "Parameter"), string.Empty, (int)MySqlDbType.String, 0);
			Parameters.Add(parameter);
			return parameter;
		}

		public virtual StiDataParameter AddParameter(string name, string expression, MySqlDbType type, int size)
		{
			var parameter = AddParameter();

			parameter.Name = name;
			parameter.Expression = expression;
			parameter.Type = (int)type;
			parameter.Size = size;

			return parameter;
		}

		public override void UpdateParameters()
		{
			if (this.DataTable == null) return;

			InvokeConnecting();

			var npgsqlAdapter = DataAdapter as MySqlDataAdapter;
			foreach (StiDataParameter parameter in Parameters)
			{
				npgsqlAdapter.SelectCommand.Parameters[parameter.Name].Value = parameter.GetParameterValue();
			}

			var dataTable = this.DataTable;
			dataTable.Rows.Clear();

			npgsqlAdapter.SelectCommand.CommandTimeout = this.CommandTimeout;
			npgsqlAdapter.Fill(dataTable);

			CheckColumnsIndexs();
		}

		public override StiDataAdapterService GetDataAdapter()
		{
			var svc = StiOptions.Services.DataAdapters.Find(x => x.GetType().FullName.Equals(typeof(StiMySqlAdapterService).FullName));
			return svc;
		}

		/// <summary>
		/// Returns new data connector for this datasource.
		/// </summary>
		/// <returns>Created connector.</returns>
		public override StiSqlDataConnector CreateConnector(string connectionString = null)
		{
			return StiMySqlConnector.Get(connectionString);
		}
		#endregion

		#region Properties
		protected override string DataAdapterType => "Reports.Stimulsoft_Data_Adapter.MySqlConnectorAdapterService";
		#endregion

		public MySqlConnectorSource()
			: this("", "", "")
		{
		}

		public MySqlConnectorSource(string nameInSource, string name)
			: this(nameInSource, name, name)
		{
		}

		public MySqlConnectorSource(string nameInSource, string name, string alias)
			: this(nameInSource, name, alias, string.Empty)
		{

		}

		public MySqlConnectorSource(string nameInSource, string name, string alias, string sqlCommand) :
			base(nameInSource, name, alias, sqlCommand)
		{
		}

		public MySqlConnectorSource(string nameInSource, string name, string alias, string sqlCommand,
			bool connectOnStart) :
			base(nameInSource, name, alias, sqlCommand, connectOnStart)
		{
		}

		public MySqlConnectorSource(string nameInSource, string name, string alias, string sqlCommand,
			bool connectOnStart, bool reconnectOnEachRow) :
			base(nameInSource, name, alias, sqlCommand, connectOnStart, reconnectOnEachRow)
		{
		}

		public MySqlConnectorSource(string nameInSource, string name, string alias, string sqlCommand,
			bool connectOnStart, bool reconnectOnEachRow, int commandTimeout) :
			base(nameInSource, name, alias, sqlCommand, connectOnStart, reconnectOnEachRow, commandTimeout)
		{
		}

		public MySqlConnectorSource(string nameInSource, string name, string alias, string sqlCommand,
			bool connectOnStart, bool reconnectOnEachRow, int commandTimeout, string key) :
			base(nameInSource, name, alias, sqlCommand, connectOnStart, reconnectOnEachRow, commandTimeout, key)
		{
		}
	}
}
